from __future__ import print_function

__title__ = "cosmid"
__version__ = "0.4.8"
__author__ = "Robin Andeer"
__license__ = "MIT"
__copyright__ = "MIT Copyright (c) 2013 Robin Andeer"
__docformat__ = "restructuredtext"
